@@ -14,7 +14,7 @@
 
 	You should have received a copy of the GNU General Public License
 	along with this program; if not, write to the Free Software
-	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 /*
@@ -74,17 +74,23 @@ struct sDirEntryList *
 	}
 	if ((tmp->sname=malloc(strlen(sname)+1))==NULL) {
 		stderror();
+		free(tmp);
 		return NULL;
 	}
 	strcpy(tmp->sname, sname);
 	if ((tmp->lname=malloc(strlen(lname)+1))==NULL) {
 		stderror();
+		free(tmp->sname);
+		free(tmp);
 		return NULL;
 	}
 	strcpy(tmp->lname, lname);
 
 	if ((tmp->sde=malloc(sizeof(struct sShortDirEntry)))==NULL) {
 		stderror();
+		free(tmp->lname);
+		free(tmp->sname);
+		free(tmp);
 		return NULL;
 	}
 	memcpy(tmp->sde, sde, DIR_ENTRY_SIZE);
@@ -110,6 +116,7 @@ struct sLongDirEntryList *
 	}
 	if ((new->lde=malloc(sizeof(struct sLongDirEntry)))==NULL) {
 		stderror();
+		free(new);
 		return NULL;
 	}
 	memcpy(new->lde, lde, DIR_ENTRY_SIZE);
@@ -164,6 +171,8 @@ int32_t cmpEntries(struct sDirEntryList *de1, struct sDirEntryList *de2) {
 	
 	char s1[MAX_PATH_LEN+1];
 	char s2[MAX_PATH_LEN+1];
+	char s1_col[MAX_PATH_LEN*2+1];
+	char s2_col[MAX_PATH_LEN*2+1];
 
 	// the volume label must always remain at the beginning of the (root) directory
 	if ((de1->sde->DIR_Atrr & (ATTR_READ_ONLY | ATTR_HIDDEN | ATTR_SYSTEM | ATTR_VOLUME_ID | ATTR_DIRECTORY)) == ATTR_VOLUME_ID) {
@@ -180,9 +189,9 @@ int32_t cmpEntries(struct sDirEntryList *de1, struct sDirEntryList *de2) {
 	} else if (strcmp(de2->sname, "..") == 0) {
 		return(1);
 	// deleted entries should be moved to the end of the directory
-	} else if (de1->sname[0] == DE_FREE) {
+	} else if ((u_char) de1->sname[0] == DE_FREE) {
 		return(1);
-	} else if (de2->sname[0] == DE_FREE) {
+	} else if ((u_char) de2->sname[0] == DE_FREE) {
 		return(-1);
 	}
 
@@ -222,6 +231,19 @@ int32_t cmpEntries(struct sDirEntryList *de1, struct sDirEntryList *de2) {
 		}
 	}
 
+	// consider last modification time
+	if (OPT_MODIFICATION) {
+		u_int32_t md1, md2;
+		md1 = de1->sde->DIR_WrtDate<<16 | de1->sde->DIR_WrtTime;
+		md2 = de2->sde->DIR_WrtDate<<16 | de2->sde->DIR_WrtTime;
+		// printf("md1: %x, md2: %x\n", md1, md2);
+		if (md1 < md2)
+				return -OPT_REVERSE;
+		else if (md1 > md2)
+				return OPT_REVERSE;
+		else return 0;
+	}
+
 	// strip special prefixes
 	if (OPT_IGNORE_PREFIXES_LIST->next != NULL) {
 		if (stripSpecialPrefixes(ss1, s1)) {
@@ -232,6 +254,15 @@ int32_t cmpEntries(struct sDirEntryList *de1, struct sDirEntryList *de2) {
 		}
 	}
 
+	// consider locale for comparison
+	if ((strxfrm(s1_col, ss1, MAX_PATH_LEN*2) == MAX_PATH_LEN*2) ||
+	    (strxfrm(s2_col, ss2, MAX_PATH_LEN*2) == MAX_PATH_LEN*2)) {
+		myerror("String collation error!");
+		exit(1);
+	}
+
+	//printf("Orig S1: %s, Orig S2: %s, Locale S1: %s, Locale S2: %s\n", ss1, ss2, s1_col, s2_col);
+
 	if (OPT_NATURAL_SORT) {
 		if (OPT_IGNORE_CASE) {
 			return natstrcasecmp(ss1, ss2) * OPT_REVERSE;
@@ -240,9 +271,9 @@ int32_t cmpEntries(struct sDirEntryList *de1, struct sDirEntryList *de2) {
 		} 
  	} else {
 		if (OPT_IGNORE_CASE) {
-			return strcasecmp(ss1, ss2) * OPT_REVERSE;
+			return strcasecmp(s1_col, s2_col) * OPT_REVERSE;
 		} else {
-			return strcmp(ss1, ss2) * OPT_REVERSE;
+			return strcmp(s1_col, s2_col) * OPT_REVERSE;
 		}
 	}
 }
@@ -319,7 +350,8 @@ void randomizeDirEntryList(struct sDirEntryList *list, u_int32_t entries) {
 	// the volume label must always remain at the beginning of the (root) directory
 	// the special "." and ".." directories must always remain at the beginning of directories, so skip them
 	while (randlist->next &&
-		(((randlist->next->sde->DIR_Atrr & (ATTR_READ_ONLY | ATTR_HIDDEN | ATTR_SYSTEM | ATTR_VOLUME_ID | ATTR_DIRECTORY)) == ATTR_VOLUME_ID) ||
+		(((randlist->next->sde->DIR_Atrr &
+		(ATTR_READ_ONLY | ATTR_HIDDEN | ATTR_SYSTEM | ATTR_VOLUME_ID | ATTR_DIRECTORY)) == ATTR_VOLUME_ID) ||
 		(strcmp(randlist->next->sname, ".") == 0) ||
 		(strcmp(randlist->next->sname, "..") == 0))) {
 
