@@ -14,7 +14,7 @@
 
 	You should have received a copy of the GNU General Public License
 	along with this program; if not, write to the Free Software
-	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 /*
@@ -32,27 +32,31 @@
 
 u_int32_t OPT_VERSION, OPT_HELP, OPT_INFO, OPT_QUIET, OPT_IGNORE_CASE,
 	OPT_ORDER, OPT_LIST, OPT_REVERSE, OPT_FORCE, OPT_NATURAL_SORT,
-	OPT_RECURSIVE, OPT_RANDOM;
-struct sStringList *OPT_INCL_DIRS, *OPT_EXCL_DIRS, *OPT_INCL_DIRS_REC, *OPT_EXCL_DIRS_REC, *OPT_IGNORE_PREFIXES_LIST;
+	OPT_RECURSIVE, OPT_RANDOM, OPT_MORE_INFO, OPT_MODIFICATION;
+struct sStringList *OPT_INCL_DIRS = NULL;
+struct sStringList *OPT_EXCL_DIRS = NULL;
+struct sStringList *OPT_INCL_DIRS_REC = NULL;
+struct sStringList *OPT_EXCL_DIRS_REC = NULL;
+struct sStringList *OPT_IGNORE_PREFIXES_LIST = NULL;
 
-int32_t addDirPathToStringList(struct sStringList *stringList, char str[MAX_PATH_LEN+1]) {
+int32_t addDirPathToStringList(struct sStringList *stringList, const char (*str)[MAX_PATH_LEN+1]) {
 /*
 	insert new string into string list
 */
 	assert(stringList != NULL);
 	assert(stringList->str == NULL);
 	assert(str != NULL);
-	assert(strlen(str) <= MAX_PATH_LEN);
+	assert(strlen((char *)str) <= MAX_PATH_LEN);
 	
 	char *newStr;
 	
 	int32_t ret, prefix=0, suffix=0, len;
 	
-	len=strlen(str);
+	len=strlen((char*)str);
 	
 	// determine whether we have to add slashes
-	if (str[0] != '/') prefix=1;
-	if (str[len-1] != '/') suffix=1;
+	if (((const char*) str)[0] != '/') prefix=1;
+	if (((const char*) str)[len-1] != '/') suffix=1;
 	
 	// allocate memory for string
 	newStr=malloc(prefix+len+suffix+1);
@@ -64,7 +68,7 @@ int32_t addDirPathToStringList(struct sStringList *stringList, char str[MAX_PATH
 	// copy string to new structure including missing slashes
 	newStr[0] = '\0';
 	strncat(newStr, "/", prefix);
-	strncat(newStr, str, len);
+	strncat(newStr, (const char*) str, len);
 	strncat(newStr, "/", suffix);
 	
 	if (prefix+len+suffix > MAX_PATH_LEN) {
@@ -85,17 +89,17 @@ int32_t matchesDirPathLists(struct sStringList *includes,
 				struct sStringList *includes_recursion,
 				struct sStringList *excludes,
 				struct sStringList *excludes_recursion,
-				char str[MAX_PATH_LEN+1]) {
+				const char (*str)[MAX_PATH_LEN+1]) {
 /*
 	evaluate whether str matches the include an exclude dir path lists or not
 */
 
 	int32_t incl, incl_rec, excl, excl_rec;
 
-	incl=matchesStringList(includes, str);
-	incl_rec=matchesStringList(includes_recursion, str);
-	excl=matchesStringList(excludes, str);
-	excl_rec=matchesStringList(excludes_recursion, str);
+	incl=matchesStringList(includes, (const char*) str);
+	incl_rec=matchesStringList(includes_recursion, (const char*) str);
+	excl=matchesStringList(excludes, (const char*) str);
+	excl_rec=matchesStringList(excludes_recursion, (const char*) str);
 	
 	// debug("str=%s,incl=%d,inclrec=%d,excl=%d,exclrec=%d", str, incl, incl_rec, excl, excl_rec);
 
@@ -126,6 +130,10 @@ int32_t parse_options(int argc, char *argv[]) {
 
 	int8_t c;
 
+	// no info by default
+	OPT_INFO = 0;
+	OPT_MORE_INFO = 0;
+
 	/* Default (1) is normal order, use -1 for reverse order. */	
 	OPT_REVERSE = 1;
 
@@ -135,22 +143,57 @@ int32_t parse_options(int argc, char *argv[]) {
 	// random sort order
 	OPT_RANDOM = 0;
 
+	// default order (directories first)
+	OPT_ORDER = 0;
+
+	// default is case sensitive
+	OPT_IGNORE_CASE = 0;
+
+	// be noisy by default
+	OPT_QUIET = 0;
+	
+	// no version information by default
+	OPT_VERSION = 0;
+
+	// sort by last modification time
+	OPT_MODIFICATION = 0;
+
 	// empty string lists for inclusion and exclusion of dirs
-	OPT_INCL_DIRS=newStringList();
-	OPT_INCL_DIRS_REC=newStringList();
-	OPT_EXCL_DIRS=newStringList();
-	OPT_EXCL_DIRS_REC=newStringList();
+	if ((OPT_INCL_DIRS=newStringList()) == NULL) {
+		myerror("Could not create stringList!");
+		return -1;
+	}
+	if ((OPT_INCL_DIRS_REC=newStringList()) == NULL) {
+		myerror("Could not create stringList!");
+		freeOptions();
+		return -1;
+	}
+	if ((OPT_EXCL_DIRS=newStringList()) == NULL) {
+		myerror("Could not create stringList!");
+		freeOptions();
+		return -1;
+	}
+	if ((OPT_EXCL_DIRS_REC=newStringList()) == NULL) {
+		myerror("Could not create stringList!");
+		freeOptions();
+		return -1;
+	}
 	
 	// empty string list for to be ignored prefixes
-	OPT_IGNORE_PREFIXES_LIST=newStringList();
+	if ((OPT_IGNORE_PREFIXES_LIST=newStringList()) == NULL) {
+		myerror("Could not create stringList!");
+		freeOptions();
+		return -1;
+	}
 
 	opterr=0;
-	while ((c=getopt(argc, argv, "ivhqcfo:lrRnd:D:x:X:I:")) != -1) {
+	while ((c=getopt(argc, argv, "imvhqcfo:lrRnd:D:x:X:I:t")) != -1) {
 		switch(c) {
 			case 'c' : OPT_IGNORE_CASE = 1; break;
 			case 'f' : OPT_FORCE = 1; break;
 			case 'h' : OPT_HELP = 1; break;
 			case 'i' : OPT_INFO = 1; break;
+			case 'm' : OPT_MORE_INFO = 1; break;
 			case 'l' : OPT_LIST = 1; break;
 			case 'o' :
 				switch(optarg[0]) {
@@ -160,36 +203,42 @@ int32_t parse_options(int argc, char *argv[]) {
 					default:
 						myerror("Unknown flag '%c' for option 'o'.", optarg[0]);
 						myerror("Use -h for more help.");
+						freeOptions();
 						return -1;
 				}
 				break;
 			case 'd' :
-				if (addDirPathToStringList(OPT_INCL_DIRS, optarg)) {
+				if (addDirPathToStringList(OPT_INCL_DIRS, (const char(*)[MAX_PATH_LEN+1]) optarg)) {
 					myerror("Could not add directory path to dirPathList");
+					freeOptions();
 					return -1;
 				}
 				break;
 			case 'D' :
-				if (addDirPathToStringList(OPT_INCL_DIRS_REC, optarg)) {
+				if (addDirPathToStringList(OPT_INCL_DIRS_REC, (const char(*)[MAX_PATH_LEN+1]) optarg)) {
 					myerror("Could not add directory path to string list");
+					freeOptions();
 					return -1;
 				}
 				break;			      
 			case 'x' :
-				if (addDirPathToStringList(OPT_EXCL_DIRS, optarg)) {
+				if (addDirPathToStringList(OPT_EXCL_DIRS, (const char(*)[MAX_PATH_LEN+1]) optarg)) {
 					myerror("Could not add directory path to string list");
+					freeOptions();
 					return -1;
 				}
 				break;
 			case 'X' :
-				if (addDirPathToStringList(OPT_EXCL_DIRS_REC, optarg)) {
+				if (addDirPathToStringList(OPT_EXCL_DIRS_REC, (const char(*)[MAX_PATH_LEN+1]) optarg)) {
 					myerror("Could not add directory path to string list");
+					freeOptions();
 					return -1;
 				}
 				break;
 			case 'I' :
 				if (addStringToStringList(OPT_IGNORE_PREFIXES_LIST, optarg)) {
 					myerror("Could not add directory path to string list");
+					freeOptions();
 					return -1;
 				}
 				break;				
@@ -197,10 +246,12 @@ int32_t parse_options(int argc, char *argv[]) {
 			case 'q' : OPT_QUIET = 1; break;
 			case 'r' : OPT_REVERSE = -1; break;
 			case 'R' : OPT_RANDOM = 1; break;
+                        case 't' : OPT_MODIFICATION = 1; break;
 			case 'v' : OPT_VERSION = 1; break;
 			default :
 				myerror("Unknown option '%c'.", optopt);
 				myerror("Use -h for more help.");
+				freeOptions();
 				return -1;
 		}
 	}
